import os
import pytest
from markdown import markdown
from pymdvar import VariableExtension


def test_empty_input():
    in_str: str = ''
    exp_str: str = markdown(in_str)
    out_str: str = markdown(in_str, extensions=[VariableExtension()])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo bar', '<p>foo bar</p>'),
    ('foo *test* bar', '<p>foo <em>test</em> bar</p>'),
    ('foo *PYMDVAR_TEST_1* bar', '<p>foo <em>PYMDVAR_TEST_1</em> bar</p>'),
    ('foo $test bar', '<p>foo $test bar</p>'),
    ('foo **$test}** bar', '<p>foo <strong>$test}</strong> bar</p>'),
    ('foo **$PYMDVAR_TEST_2}** bar', '<p>foo <strong>$PYMDVAR_TEST_2}</strong> bar</p>'),
    ('foo [link](${test/a.html) bar', '<p>foo <a href="${test/a.html">link</a> bar</p>'),
    ('foo ![image]($test}/a.jpg) bar', '<p>foo <img alt="image" src="$test}/a.jpg" /> bar</p>')
])
def test_no_replacements_config(in_str: str, exp_str: str, single_var_dict: dict[str, str]):
    out_str: str = markdown(in_str, extensions=[VariableExtension(variables=single_var_dict, enable_env=True)])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo *${test}* bar', '<p>foo <em></em> bar</p>'),
    ('foo *${PYMDVAR_TEST_1}* bar', '<p>foo <em></em> bar</p>'),
    ('foo [link](${test}/a.html) bar', '<p>foo <a href="/a.html">link</a> bar</p>'),
])
def test_replacements_no_config(in_str: str, exp_str: str):
    out_str: str = markdown(in_str, extensions=[VariableExtension()])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo *${test}* bar', '<p>foo <em>value</em> bar</p>'),
    ('foo *${PYMDVAR_TEST_1}* bar', '<p>foo <em>1</em> bar</p>'),
    ('foo ${value} bar', '<p>foo  bar</p>'),
    ('foo **${PYMDVAR_TEST_2}** bar', '<p>foo <strong></strong> bar</p>'),
    ('foo [link](${test}/a.html) bar', '<p>foo <a href="value/a.html">link</a> bar</p>'),
    ('foo ![image](${PYMDVAR_TEST_1}/a.jpg) bar', '<p>foo <img alt="image" src="1/a.jpg" /> bar</p>'),
    ('foo [link](${PYMDVAR_TEST_2}/a.html) bar', '<p>foo <a href="/a.html">link</a> bar</p>')
])
def test_simple_replacements(in_str: str, exp_str: str, single_var_dict: dict[str, str]):
    out_str: str = markdown(in_str, extensions=[VariableExtension(variables=single_var_dict, enable_env=True)])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo *${test}* bar', '<p>foo <em>value</em> bar</p>'),
    ('foo *${TEST}* bar', '<p>foo <em>VALUE</em> bar</p>'),
    ('foo **${SOMETHING_ELSE}** bar', '<p>foo <strong>something_else</strong> bar</p>')
])
def test_multivar_replacements(in_str: str, exp_str: str, multi_var_dict: dict[str, str]):
    out_str: str = markdown(in_str, extensions=[VariableExtension(variables=multi_var_dict)])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo *${test}* and **${TEST}** bar', '<p>foo <em>value</em> and <strong>VALUE</strong> bar</p>'),
    ('foo ![image](${PYMDVAR_TEST_1}/a.${ext}) bar', '<p>foo <img alt="image" src="/a.jpg" /> bar</p>')
])
def test_text_multivar_replacements(in_str: str, exp_str: str, multi_var_dict: dict[str, str]):
    out_str: str = markdown(in_str, extensions=[VariableExtension(variables=multi_var_dict)])
    assert out_str == exp_str


@pytest.mark.parametrize('in_str, exp_str', [
    ('foo *${test}* and **${TEST}** bar', '<p>foo <em>value</em> and <strong>VALUE</strong> bar</p>'),
    ('foo ![image](${PYMDVAR_TEST_1}/a.${ext}) bar', '<p>foo <img alt="image" src="/a.jpg" /> bar</p>')
])
def test_extension_text_mode(in_str: str, exp_str: str, multi_var_dict: dict[str, str]):
    out_str: str = markdown(in_str, extensions=['pymdvar'], extension_configs={'pymdvar': {'variables': multi_var_dict}})
    assert out_str == exp_str



import pytest


@pytest.fixture
def single_var_dict():
    return {'test': 'value'}


@pytest.fixture
def multi_var_dict():
    return {'ext': 'jpg',
            'test': 'value',
            'TEST': 'VALUE',
            'SOMETHING_ELSE': 'something_else'}



from typing import Any
from .pymdvar import VariableExtension

__all__ = ['VariableExtension']


# this should be in pymdvar.py, but since there is only one extension,
#  it is fine in here
def makeExtension(*args: Any, **kwargs: Any):
    return VariableExtension(*args, **kwargs)



import os
from re import Match
from typing import Any
from xml.etree.ElementTree import Element
from markdown import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import Pattern

VAR_RE: str = r'(\$\{)([a-zA-Z_0-9]*)(\})'


class VarPattern(Pattern):
    # need to redefine as an extra attribute needs to be passed
    def __init__(self, pattern: Any,
                 vars: dict[str, str],
                 enable_env: bool,
                 md: Markdown | None = None) -> None:
        self.vars: dict[str, str] = vars
        self.enable_env: bool = enable_env
        super().__init__(pattern, md)

    def handleMatch(self, m: Match[str]) -> str | Element | None:
        # for some reason the group is offest by 1
        var: str | Any = m.group(3)
        if var in self.vars:
            return self.vars[var]
        if self.enable_env:
            if var in os.environ:
                return os.environ[var]
        return ''


class VariableExtension(Extension):
    def __init__(self, **kwargs: Any) -> None:
        self.config: dict[str, list[Any | str]] = {
            'enable_env': [False, 'Enable environment variables parsing.'],
            'variables': [dict(), 'Dictionary holding variables to be used.']
        }
        super().__init__(**kwargs)

    def extendMarkdown(self, md: Markdown) -> None:
        vars: dict[str, str] | Any = self.getConfig('variables')
        enable_env: bool = self.getConfig('enable_env')
        var_pattern: VarPattern = VarPattern(VAR_RE, vars, enable_env)
        md.inlinePatterns.register(var_pattern, 'variable', 175)
